<?php

namespace Unific\Connector\Helper\Data;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Order extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $filterHelper;

    protected $customerSession;
    protected $dataObjectConverter;

    // Holds a Order API DATA object
    protected $order;

    protected $returnData = [];

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Connector\Helper\Filter $filterHelper
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\Api\ExtensibleDataObjectConverter $dataObjectConverter
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Connector\Helper\Filter $filterHelper,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\Api\ExtensibleDataObjectConverter $dataObjectConverter
    ) {
        parent::__construct($context);

        $this->filterHelper = $filterHelper;

        $this->customerSession = $customerSession;
        $this->dataObjectConverter = $dataObjectConverter;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     */
    public function setOrder(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $this->order = $order;
        $this->setOrderInfo();
    }

    /**
     * @return \Magento\Sales\Model\Data\OrderInterface
     */
    public function getOrder()
    {
        return $this->order;
    }

    /**
     * @return void
     */
    protected function setOrderInfo()
    {
        $this->returnData = $this->order->getData();

        $this->returnData['order_items'] = [];
        foreach ($this->order->getAllItems() as $item) {
            $itemData = $item->getData();

            $itemData['free_shipping'] = (isset($itemData['free_shipping']) && $itemData['free_shipping'] == true) ? 1 : 0;

            if (isset($itemData['is_qty_decimal'])) {
                $itemData['is_qty_decimal'] = (int)$itemData['is_qty_decimal'];
            }

            if (isset($itemData['qty_ordered'])) {
                $itemData['qty_ordered'] = (int)$itemData['qty_ordered'];
            }

            if (isset($itemData['qty_canceled'])) {
                $itemData['qty_canceled'] = (int)$itemData['qty_canceled'];
            }

            if (isset($itemData['qty_invoiced'])) {
                $itemData['qty_invoiced'] = (int)$itemData['qty_invoiced'];
            }

            if (isset($itemData['qty_refunded'])) {
                $itemData['qty_refunded'] = (int)$itemData['qty_refunded'];
            }

            if (isset($itemData['qty_shipped'])) {
                $itemData['qty_shipped'] = (int)$itemData['qty_shipped'];
            }

            $this->returnData['order_items'][] = array_intersect_key($itemData, array_flip($this->filterHelper->getItemsWhitelist()));
        }

        $this->returnData['addresses'] = [];

        if($this->order->getBillingAddress() !== null)
        {
            $this->returnData['addresses']['billing'] = $this->dataObjectConverter->toFlatArray($this->order->getBillingAddress(), [], \Magento\Sales\Api\Data\OrderAddressInterface::class);
            $this->returnData['addresses']['billing'] = $this->filterHelper->fixAddressKey($this->returnData['addresses']['billing'], 'street');
        }

        if($this->order->getShippingAddress() !== null)
        {
            $this->returnData['addresses']['shipping'] = $this->dataObjectConverter->toFlatArray($this->order->getShippingAddress(), [], \Magento\Sales\Api\Data\OrderAddressInterface::class);
            $this->returnData['addresses']['shipping'] = $this->filterHelper->fixAddressKey($this->returnData['addresses']['shipping'], 'street');
        }

        $this->returnData['payment'] = $this->order->getPayment()->getData();
    }

    /**
     * @return array
     */
    public function getOrderInfo()
    {
        // Sanitize order
        $this->returnData = array_intersect_key($this->returnData, array_flip($this->filterHelper->getOrderWhiteList()));

        if(isset($this->returnData['customer_is_guest'])) {
            $this->returnData['customer_is_guest'] = intval($this->returnData['customer_is_guest']);
        }

        // Sanitize order payment
        if(isset($this->returnData['payment'])) {
            $this->returnData['payment'] = array_intersect_key($this->returnData['payment'], array_flip($this->filterHelper->getPaymentWhitelist()));
        }

        for ($i = 0; $i < count($this->returnData['order_items']); $i++) {
            $this->returnData['order_items'][$i] = array_intersect_key($this->returnData['order_items'][$i], array_flip($this->filterHelper->getItemsWhitelist()));
        }

        return $this->filterHelper->sanitizeAddressData($this->returnData);
    }
}
