<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;

class CheckoutEnrichPlugin extends AbstractPlugin
{
    protected $cartDataHelper;
    protected $cartRepository;
    protected $quoteIdMaskFactory;
    protected $customerSession;
    protected $customerDataHelper;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Cart $cartDataHelper
     * @param \Unific\Connector\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory
     * @param \Magento\Customer\Model\Session $customerSession
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Cart $cartDataHelper,
        \Unific\Connector\Helper\Data\Customer $customerDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory,
        \Magento\Customer\Model\Session $customerSession
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;
        $this->quoteIdMaskFactory = $quoteIdMaskFactory;
        $this->customerSession = $customerSession;
        $this->customerDataHelper = $customerDataHelper;
    }

    /**
     * @param $subject
     * @param callable $proceed
     * @param $cartId
     * @param \Magento\Quote\Model\Quote\Address $address
     * @return array
     */
    public function aroundEstimateByExtendedAddress($subject, callable $proceed, $cartId, \Magento\Quote\Model\Quote\Address $address)
    {
        $returnValue = $proceed($cartId, $address);

        if ($this->scopeConfig->getValue('unific/connector/enabled')  == 1) {
            if (!$subject instanceof \Magento\Quote\Model\ShippingMethodManagement) {
                $integrationSubject = 'checkout/update';

                $quoteIdMask = $this->quoteIdMaskFactory->create()->load($cartId, 'masked_id');

                if($quoteIdMask->getQuoteId() !== null) {
                    $this->cartDataHelper->setCart($this->cartRepository->get($quoteIdMask->getQuoteId()), $cartId);

                    if ($address->getFirstname() != null) {
                        $this->cartDataHelper->setAddressData($address);
                    }

                    $this->processWebhook(
                        $this->cartDataHelper->getCartInfo(),
                        $this->scopeConfig->getValue('unific/webhook/cart_endpoint'),
                        Settings::PRIORITY_CART,
                        $integrationSubject
                    );
                }
            }
        }

        // Send customer
        if ($this->scopeConfig->getValue('unific/connector/enabled') && $this->customerSession->isLoggedIn() === false) {
            // Send a customer update, if not logged in, generate a customer from quote data
            $quoteIdMask = $this->quoteIdMaskFactory->create()->load($cartId, 'masked_id');

            if($quoteIdMask->getQuoteId() !== null) {
                if($this->customerSession->isLoggedIn() === false) {
                    $this->customerDataHelper->generateGuestCustomer($this->cartRepository->get($quoteIdMask->getQuoteId()));
                } else {
                    $this->customerDataHelper->setCustomer($this->customerSession->getCustomer()->getDataModel());
                }

                $customerData = $this->customerDataHelper->getCustomerInfo();

                if($customerData['email'] != null && filter_var($customerData['email'], FILTER_VALIDATE_EMAIL)) {
                    $this->processWebhook(
                        $customerData,
                        $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                        Settings::PRIORITY_CUSTOMER,
                        'customer/update'
                    );
                }
            }
        }

        return $returnValue;
    }
}
