<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;
use Magento\Customer\Api\CustomerRepositoryInterface as CustomerRepository;
use Magento\Customer\Api\Data\CustomerInterface;

class CustomerPlugin extends AbstractPlugin
{
    protected $customerDataHelper;
    protected $customerRepository;

    protected $request;
    protected $subscriberFactory;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Customer $customerDataHelper,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->customerDataHelper = $customerDataHelper;
        $this->customerRepository = $customerRepository;

        $this->request = $request;
        $this->subscriberFactory = $subscriberFactory;
    }

    /**
     *
     * @param CustomerRepository $subject
     * @param CustomerInterface $result
     * @param CustomerInterface $customer
     * @return CustomerInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterSave(CustomerRepository $subject, CustomerInterface $result, CustomerInterface $customer)
    {
        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1 && $result->getId() > 0) {
            $webhookSubject = 'customer/create';

            // Only send if the date created is more than 5 seconds different from the updated date
            // Sometimes a second save action happens within a second after creation
            if ($result->getUpdatedAt() != "" && strtotime($result->getUpdatedAt()) - strtotime($result->getCreatedAt()) > 5) {
                $webhookSubject = 'customer/update';
            }

            $this->customerDataHelper->setCustomer($result);

            if (filter_var($result->getEmail(), FILTER_VALIDATE_EMAIL)) {
                $subscriber = $this->subscriberFactory->create()->loadByEmail($result->getEmail());
                $this->customerDataHelper->setOptinNewsletter($this->request->getParam('is_subscribed', $subscriber->isSubscribed()));

                $this->processWebhook(
                    $this->customerDataHelper->getCustomerInfo(),
                    $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                    Settings::PRIORITY_CUSTOMER,
                    $webhookSubject
                );
            }
        }

        return $result;
    }
}
