<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;

class OrderPlugin extends AbstractPlugin
{
    protected $orderDataHelper;
    protected $cartDataHelper;
    protected $cartRepository;
    protected $customerSession;
    protected $customerDataHelper;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Order $orderDataHelper
     * @param \Unific\Connector\Helper\Data\Cart $cartDataHelper
     * @param \Unific\Connector\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Customer\Model\Session $customerSession
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Order $orderDataHelper,
        \Unific\Connector\Helper\Data\Cart $cartDataHelper,
        \Unific\Connector\Helper\Data\Customer $customerDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Customer\Model\Session $customerSession
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;

        $this->orderDataHelper = $orderDataHelper;
        $this->customerSession = $customerSession;
        $this->customerDataHelper = $customerDataHelper;
    }

    /**
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $entity
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    public function afterSave($subject, \Magento\Sales\Api\Data\OrderInterface $entity)
    {
        $integrationSubject = 'order/create';

        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1) {
            switch ($entity->getState()) {
                case \Magento\Sales\Model\Order::STATE_PROCESSING:
                    $integrationSubject = 'order/update';
                    break;

                case \Magento\Sales\Model\Order::STATE_CANCELED:
                    $integrationSubject = 'order/cancel';
                    break;

                case \Magento\Sales\Model\Order::STATE_COMPLETE:
                    $integrationSubject = 'order/ship';
                    break;

                case \Magento\Sales\Model\Order::STATE_CLOSED:
                    $integrationSubject = 'order/refund';
                    break;
            }

            // Send the order
            $this->orderDataHelper->setOrder($entity);
            $this->processWebhook(
                $this->orderDataHelper->getOrderInfo(),
                $this->scopeConfig->getValue('unific/webhook/order_endpoint'),
                Settings::PRIORITY_ORDER,
                $integrationSubject
            );
        }

        // Send the cart
        if ($this->scopeConfig->getValue('unific/connector/enabled') && $integrationSubject == 'order/create') {
            if($entity->getQuoteId() != null) {
                $this->cartDataHelper->setCart($this->cartRepository->get($entity->getQuoteId()));
                $this->processWebhook(
                    $this->cartDataHelper->getCartInfo(),
                    $this->scopeConfig->getValue('unific/webhook/cart_endpoint'),
                    Settings::PRIORITY_CART,
                    'checkout/update'
                );
            }
        }

        // Send customer
        if ($this->scopeConfig->getValue('unific/connector/enabled') && $integrationSubject == 'order/create') {
            // Send a customer update, if not logged in, generate a customer from order data
            if ($this->customerSession->isLoggedIn()) {
                $this->customerDataHelper->setCustomer($this->customerSession->getCustomer()->getDataModel());
            } else {
                $this->customerDataHelper->generateGuestCustomer($entity);
            }

            $customerData = $this->customerDataHelper->getCustomerInfo();

            if($customerData['email'] != null && filter_var($customerData['email'], FILTER_VALIDATE_EMAIL)) {
                $this->processWebhook(
                    $this->customerDataHelper->getCustomerInfo(),
                    $this->scopeConfig->getValue('unific/webhook/customer_endpoint'),
                    Settings::PRIORITY_CUSTOMER,
                    'customer/update'
                );
            }
        }

        return $entity;
    }
}
