<?php

namespace Unific\Connector\Plugin;

use Unific\Connector\Helper\Settings;

class ProductPlugin extends AbstractPlugin
{
    protected $productDataHelper;

    /**
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Unific\Connector\Helper\Hmac $hmacHelper
     * @param \Unific\Connector\Helper\Message\Queue $queueHelper
     * @param \Magento\Framework\App\ProductMetadataInterface $productMetadata
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Unific\Connector\Helper\Data\Product $productDataHelper
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Unific\Connector\Helper\Hmac $hmacHelper,
        \Unific\Connector\Helper\Message\Queue $queueHelper,
        \Magento\Framework\App\ProductMetadataInterface $productMetadata,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Unific\Connector\Helper\Data\Product $productDataHelper
    ) {
        parent::__construct(
            $scopeConfig,
            $hmacHelper,
            $queueHelper,
            $productMetadata,
            $moduleList
        );

        $this->productDataHelper = $productDataHelper;
    }

    /**
     * @param \Magento\Catalog\Model\Product $subject
     * @return \Magento\Catalog\Model\Product
     */
    public function afterSave(\Magento\Catalog\Model\Product $subject)
    {
        if ($this->scopeConfig->getValue('unific/connector/enabled') == 1) {
            $integrationSubject = 'product/create';

            if ($subject->getCreatedAt() != $subject->getUpdatedAt() && $subject->getUpdatedAt() != null) {
                $integrationSubject = 'product/update';
            }

            $this->productDataHelper->setProduct($subject);
            $this->processWebhook(
                $this->productDataHelper->getProductInfo(),
                $this->scopeConfig->getValue('unific/webhook/product_endpoint'),
                Settings::PRIORITY_PRODUCT,
                $integrationSubject
            );
        }

        return $subject;
    }
}
